Package: request
Title: High Level 'HTTP' Client
Description: High level and easy 'HTTP' client for 'R'. Provides functions for
    building 'HTTP' queries, including query parameters, body requests, headers,
    authentication, and more.
Version: 0.1.0
Authors@R: person("Scott", "Chamberlain", role = c("aut", "cre"),
    email = "myrmecocystus@gmail.com")
License: MIT + file LICENSE
URL: https://github.com/sckott/request
BugReports: https://github.com/sckott/request/issues
Depends: httr (>= 1.0.0)
Imports: methods, stats, utils, curl (>= 0.9.4), jsonlite (>= 0.9.19),
        magrittr (>= 1.5), lazyeval (>= 0.1.10), whisker (>= 0.3-2), R6
        (>= 2.1.1)
Suggests: testthat
RoxygenNote: 5.0.1
NeedsCompilation: no
Packaged: 2016-01-02 21:26:39 UTC; sacmac
Author: Scott Chamberlain [aut, cre]
Maintainer: Scott Chamberlain <myrmecocystus@gmail.com>
Repository: CRAN
Date/Publication: 2016-01-03 16:14:12
Built: R 4.6.0; ; 2026-04-28 09:44:48 UTC; unix
