# Generated by roxygen2: do not edit by hand

S3method(print,endpoint)
S3method(print,req)
S3method(type_sum,Date)
S3method(type_sum,POSIXt)
S3method(type_sum,array)
S3method(type_sum,character)
S3method(type_sum,default)
S3method(type_sum,factor)
S3method(type_sum,integer)
S3method(type_sum,logical)
S3method(type_sum,matrix)
S3method(type_sum,numeric)
export("%>%")
export(api)
export(api_body)
export(api_body_)
export(api_config)
export(api_error_handler)
export(api_headers)
export(api_headers_)
export(api_oauth1)
export(api_oauth2)
export(api_path)
export(api_path_)
export(api_query)
export(api_query_)
export(api_simple_auth)
export(api_template)
export(api_write)
export(http)
export(http_client)
export(peep)
export(type_sum)
import(httr)
importFrom(R6,R6Class)
importFrom(curl,curl_version)
importFrom(lazyeval,all_dots)
importFrom(lazyeval,lazy_dots)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(stats,setNames)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(whisker,whisker.render)
